﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

[RequireComponent(typeof(AudioSource))]
[RequireComponent(typeof(Rigidbody))]
[DisallowMultipleComponent]
public class RocketController : MonoBehaviour {

    //Cached
    Rigidbody myRigidBody;
    AudioSource myAudioSource;

    //Particle systems
    [Header("Particle Systems")]
    [SerializeField] ParticleSystem thrustParticles;
    [SerializeField] ParticleSystem successParticles;
    [SerializeField] ParticleSystem explosionParticles;

    //AudioClips
    [Header("AudioClips")]
    [SerializeField] AudioClip thrustAudio;
    [SerializeField] AudioClip successAudio;
    [SerializeField] AudioClip explosionAudio;

    //Configuration parameters
    [Header("Configuration Parameters")]
    [Range(5f, 200f)][SerializeField] float thrustForce = 30f;
    [Range(10f, 1000f)][SerializeField] float rotationSpeed = 350f;
    [Range(0f, 60f)][SerializeField] float sceneLoadDelay = 3f;
    [Range(0f, 1f)] [SerializeField] float crashBounciness = 0.5f;

    //State
    enum State { alive, dying, transcending };
    State currentState;
    int currentSceneIndex;

    void Awake()
    {
        currentSceneIndex = SceneManager.GetActiveScene().buildIndex;
    }

    void Start ()
    {
        currentState = State.alive;
        myRigidBody = GetComponent<Rigidbody>();
        myAudioSource = GetComponent<AudioSource>();
        myAudioSource.volume = 0f;
	}
	
	void Update ()
    {
        if (currentState == State.alive)
        {
            ProcessInput();
        }
        else
        {
            StartCoroutine(FadeThrustAudio());
        }
    }

    private void ProcessInput()
    {
        RespondToThrustInput();
        RespondToRotationInput();
    }

    private void RespondToThrustInput()
    {
        if (Input.GetKey(KeyCode.Space))
        {
            ApplyThrust();
        }
        else
        {
            StartCoroutine(FadeThrustAudio());
            thrustParticles.Stop();
        }
    }

    private void ApplyThrust()
    {
        myRigidBody.AddRelativeForce(Vector3.up * thrustForce);

        if (myAudioSource.isPlaying && myAudioSource.volume == 0f)
        {
            myAudioSource.Stop();
            myAudioSource.volume = 1f;
        }
        else if (!myAudioSource.isPlaying)
        {
            myAudioSource.volume = 1f;
            myAudioSource.PlayOneShot(thrustAudio);
        }
        thrustParticles.Play();
    }

    private void RespondToRotationInput()
    {
        myRigidBody.freezeRotation = true;
        
        if (Input.GetKey(KeyCode.A) || Input.GetKey(KeyCode.LeftArrow))
        {
            transform.Rotate(Vector3.forward * rotationSpeed * Time.deltaTime);
        }
        else if (Input.GetKey(KeyCode.D) || Input.GetKey(KeyCode.RightArrow))
        {
            transform.Rotate(-Vector3.forward * rotationSpeed * Time.deltaTime);
        }
        
        myRigidBody.freezeRotation = false;
    }

    private void OnCollisionEnter(Collision collision)
    {
        if(currentState != State.alive) { return; }

        switch (collision.gameObject.tag)
        {
            case "Friendly":
                // do nothing
                break;

            case "Finish":
                StartSuccessSequence();
                break;

            case "Fuel":
                print("Increase fuel");
                break;

            default:
                StartDeathSequence();
                break;
        }
    }

    private void StartSuccessSequence()
    {
        currentState = State.transcending;
        thrustParticles.Stop();
        successParticles.Play();
        AudioSource.PlayClipAtPoint(successAudio, Camera.main.transform.position, 0.3f);
        Invoke("LoadNextScene", sceneLoadDelay);
    }

    private void StartDeathSequence()
    {
        currentState = State.dying;
        thrustParticles.Stop();
        explosionParticles.Play();
        foreach (Transform child in transform)
        {
            if (child.gameObject.GetComponent<BoxCollider>())
            {
                child.gameObject.AddComponent<Rigidbody>();
                child.gameObject.GetComponent<Rigidbody>().useGravity = false;
                child.gameObject.GetComponent<Rigidbody>().AddExplosionForce(1500f, transform.position, 100f);
                child.gameObject.GetComponent<BoxCollider>().material.bounciness = crashBounciness;
            }
        }
        transform.DetachChildren();
        AudioSource.PlayClipAtPoint(explosionAudio, Camera.main.transform.position, 0.7f);
        Invoke("LoadCurrentScene", sceneLoadDelay);
    }

    private void LoadNextScene()
    {
        SceneManager.LoadScene(currentSceneIndex + 1);
    }

    private void LoadCurrentScene()
    {

        SceneManager.LoadScene(currentSceneIndex);
    }

    private IEnumerator FadeThrustAudio()
    {
        while(myAudioSource.volume > 0f)
        {
            myAudioSource.volume -= Time.deltaTime / 0.25f;
            yield return null;
        }

        myAudioSource.volume = 0f;
    }
}
